@@ -43,6 +43,9 @@ class SQLiteHandler(object):
         else:
             self.database_init(filename=filename, path=path)
 
+    def database_saveas(self, filename, path=""):
+        self.database.saveas_database(filename=filename, path=path)
+
     def database_open(self, filename, path=""):
         """
         initialises database only if it already exists
@@ -74,21 +77,23 @@ class SQLiteHandler(object):
         else:
             print(f"Couldn't delete database, not connected to one!")
 
-    def table_sync(self, table):
+    def table_sync(self, tablename):
         
-        sqlrecords = self.database.read_records(tablename=table.name, columns=table.column_names)
+        table = self.database.tables[tablename]
+        sqlrecords = self.database.read_records(tablename=tablename, columns=table.column_names)
         table.records = self.transform_sql_to_record(table=table, sqlrecords=sqlrecords)
 
-    def table_add_records(self, table, records):
+    def table_add_records(self, tablename, records):
 
+        table = self.database.tables[tablename]
         self.database.add_records(
             table,
             records,
         )
 
-        self.table_sync(table)
+        self.table_sync(tablename)
 
-    def table_update_records(self, table, valuepairs, where):
+    def table_update_records(self, tablename, valuepairs, where):
         """
         update specific records of the specified table
 
@@ -101,20 +106,25 @@ class SQLiteHandler(object):
             where = [["id", [where]]]
 
         self.database.update_records(
-            tablename=table.name, 
+            tablename=tablename, 
             valuepairs = valuepairs,
             where=where,
         )
 
-    def table_read_records(self, table, where=[]):
+    def table_read_records(self, tablename, where=[]):
 
-        sqlrecords = self.database.read_records(tablename=table.name, columns=table.column_names, where=where)
-        records = self.transform_sql_to_record(table=table, sqlrecords=sqlrecords)
-        # print(f"{self.name} records retrieved: {self.records}")
+        table = self.database.tables[tablename]
+
+        if where == []:
+            records = table.records
+        else:
+            sqlrecords = self.database.read_records(tablename=tablename, columns=table.column_names, where=where)
+            records = self.transform_sql_to_record(table=table, sqlrecords=sqlrecords)
+            # print(f"{self.name} records retrieved: {self.records}")
 
         return records
 
-    def table_read_foreign_records(self, table, column, where=[]):
+    def table_read_foreign_records(self, tablename, column, where=[]):
         """
         for a particular tableobject and column,
         checks if column is a foreign key
@@ -123,6 +133,8 @@ class SQLiteHandler(object):
         records will be returned as an array of record objects
         """
 
+        table = self.database.tables[tablename]
+
         # check if the column points to a foreign key
         columnindex = table.column_names.index(column)
         split = table.column_types[columnindex].split(' ', 3)
@@ -141,7 +153,7 @@ class SQLiteHandler(object):
         print(f"record objects of foreign table {recordobjects}")
 
         # get the foreign keys to filter on
-        records = self.table_read_records(table=table, where=where)
+        records = self.table_read_records(tablename=tablename, where=where)
         foreign_keys = []
         for record in records:
             foreign_keys += [record.recorddict[column]]
@@ -191,14 +203,14 @@ class SQLiteHandler(object):
         return the found table
         """
 
-        for key in self.database.tables:
-            if ("CROSSREF" in key) and (tablename1 in key) and (tablename2 in key):
-                table = self.database.tables[key]
+        for tablename in self.database.tables:
+            if ("CROSSREF" in tablename) and (tablename1 in tablename) and (tablename2 in tablename):
+                table = self.database.tables[tablename]
                 break
 
         return table
 
-    def crossref_add_record(self, table1, table2, where1, where2, description=""):
+    def crossref_add_record(self, tablename1, tablename2, where1, where2, description=""):
         """
         adds a crossreference between tables 1 and 2 for the rows given by the where statements.
         Creates links for any combination of the found rows of table1 and table 2. So if the where statements point to multiple rows,
@@ -219,10 +231,13 @@ class SQLiteHandler(object):
         3 - 8
         """
 
+        table1 = self.database.tables[tablename1]
+        table2 = self.database.tables[tablename2]
+
         # get the cross reference table
         crossref = self.crossref_get_one(
-            tablename1 = table1.name,
-            tablename2 = table2.name, 
+            tablename1 = tablename1,
+            tablename2 = tablename2, 
             )
 
         # print(f"where1 is {where1}")
@@ -232,7 +247,7 @@ class SQLiteHandler(object):
         else:
             rowids1 = []
             records = self.table_read_records(
-                table = table1,
+                tablename = tablename1,
                 where = where1,
             )
             for record in records:
@@ -245,7 +260,7 @@ class SQLiteHandler(object):
         else:
             rowids2 = []
             records = self.table_read_records(
-                table = table2,
+                tablename = tablename2,
                 where = where2,
             )
             for record in records:
@@ -253,11 +268,11 @@ class SQLiteHandler(object):
         # print(f"rowids2 is {rowids2}")
 
         # determine if tables need to be switched places
-        index1 = crossref.name.find(table1.name)
-        index2 = crossref.name.find(table2.name)
+        index1 = crossref.name.find(tablename1)
+        index2 = crossref.name.find(tablename2)
 
         if (index1 == -1) or (index2 == -1):
-            print(f"table1 {table1.name} or table2 {table2.name} not found")
+            print(f"table1 {tablename1} or table2 {tablename2} not found")
             return
 
         # switch columns
@@ -272,19 +287,19 @@ class SQLiteHandler(object):
                     description = f"Cross referenced {where1} to {where2}"
 
                 record = self.record_create(
-                    table=crossref,
+                    tablename=crossref.name,
                     values=[
                         value1, value2, description
                     ]
                 )
                 records += [record]
 
-        self.table_add_records(crossref, records)
+        self.table_add_records(crossref.name, records)
 
     def crossref_read_records(self, tablename1, tablename2):
 
-        table = self.crossref_get_one(tablename1, tablename2)
-        records = self.table_read_records(table)
+        table = self.crossref_get_one(tablename1=tablename1, tablename2=tablename2)
+        records = self.table_read_records(table.name)
 
         return records
 
@@ -295,7 +310,7 @@ class SQLiteHandler(object):
 
         table = self.crossref_get_one(tablename1, tablename2)
         # print(table.name)
-        records = self.table_read_records(table)
+        records = self.table_read_records(table.name)
         
         # print(records)
 
@@ -309,18 +324,22 @@ class SQLiteHandler(object):
 
         return records
 
-    def record_create(self, table, values):
+    def record_create(self, tablename, values=[]):
         """
         creates a draft record that can still be 
         manipulated before making it definitive
+        if values are empty, takes default values of the columns
 
         can be added to the table through
         table_add_records method
         """
 
-        lastrow = self.database.get_max_row(table.name)
+        table = self.database.tables[tablename]
 
-        recordarray = [lastrow + 1] + values
+        if values == []:
+            recordarray = table.defaults
+        else:
+            recordarray = [-1] + values
 
         record = Record(
             table.column_names,
@@ -329,7 +348,7 @@ class SQLiteHandler(object):
         print(f"created record with recordarray {record.recordarray}")
         return record
 
-    def records_create(self, table, recordsvalues):
+    def records_create(self, tablename, recordsvalues):
         """
         creates multiple draft records that can still be 
         manipulated before making it definitive
@@ -340,11 +359,14 @@ class SQLiteHandler(object):
 
         records = []
         for values in recordsvalues:
-            records += [self.record_create(table, values)]
+            records += [self.record_create(tablename, values)]
 
         return records
 
     def transform_sql_to_record(self, table, sqlrecords):
+        """
+        uses table object as input, as its used privately and not made to be called directly by using application
+        """
 
         # print(sqlrecords)
 
@@ -385,64 +407,64 @@ if __name__ == "__main__":
     # adding multiple records
     records = []
     records += [handler.record_create(
-        table_scientists,
-        [1, "Hawking", 68, True],
+        tablename="scientists",
+        values=[1, "Hawking", 68, True],
         )]
     records += [handler.record_create(
-        table_scientists,
-        [2, "Edison's child said \"Apple!\"", 20, True],
+        tablename="scientists",
+        values=[2, "Edison's child said \"Apple!\"", 20, True],
         )]
-    handler.table_add_records(table_scientists, records)
+    handler.table_add_records(tablename="scientists", records=records)
     print(f"creating multiple records")
-    print_records(table_scientists.records)
+    print_records(handler.table_read_records(tablename="scientists"))
     
     # adding single records
     records = []
     records += [handler.record_create(
-        table_scientists,
-        [3, "Einstein", 100, False],
+        tablename="scientists",
+        values=[3, "Einstein", 100, False],
         )]
-    handler.table_add_records(table_scientists, records)
+    handler.table_add_records(tablename="scientists", records=records)
     print(f"creating single records")
-    print_records(table_scientists.records)
+    print_records(handler.table_read_records(tablename="scientists"))
 
     # adding multiple records
     records = []
     records += [handler.record_create(
-        table_scientists,
-        [4, "Rosenburg", 78, False],
+        tablename="scientists",
+        values=[4, "Rosenburg", 78, False],
         )]
     records += [handler.record_create(
-        table_scientists,
-        [5, "Neil dGrasse Tyson", 57, True],
+        tablename="scientists",
+        values=[5, "Neil dGrasse Tyson", 57, True],
         )]
-    handler.table_add_records(table_scientists, records)
+    handler.table_add_records(tablename="scientists", records=records)
     print(f"creating multiple records")
-    print_records(table_scientists.records)
+    print_records(handler.table_read_records(tablename="scientists"))
 
     # conditional read with where statement
     where = [["nobelprizewinner", [True]]]
-    records = handler.table_read_records(table=table_scientists, where=where)
+    records = handler.table_read_records(tablename="scientists", where=where)
     print(f"read where")
     print_records(records)
 
     # update with where statement
     valuepairs = [["nobelprizewinner", False]]
     where = [["nobelprizewinner", [True]], ["name", ["Hawking"]]]
-    handler.table_update_records(table=table_scientists, valuepairs=valuepairs, where=where)
-    records = handler.table_read_records(table=table_scientists)
+    handler.table_update_records(tablename="scientists", valuepairs=valuepairs, where=where)
+    records = handler.table_read_records(tablename="scientists")
     print(f"update true to false")
     print_records(records)
 
     valuepairs = [["name", "Neil de'Grasse Tyson"], ["age", 40]]
     rowid = 5
-    handler.table_update_records(table=table_scientists, valuepairs=valuepairs, where=rowid)
-    records = handler.table_read_records(table=table_scientists)
+    handler.table_update_records(tablename="scientists", valuepairs=valuepairs, where=rowid)
+    records = handler.table_read_records(tablename="scientists")
     print(f"update record 'id = 5'")
     print_records(records)
 
     # get records without table object, but with tablename
-    records = handler.table_read_records(table_scientists)
+    records = handler.table_read_records(tablename="scientists")
     print(f"read all records")
     print_records(records)
 
@@ -460,31 +482,31 @@ if __name__ == "__main__":
     
     # adding multiple records in one go
     records = handler.records_create(
-        table_nobel,
-            [
+        tablename="nobelprizes",
+        recordsvalues=[
             [1, "Peace", "Peace nobel prize"],
             [2, "Economy", "Economy nobel prize"],
             [3, "Physics", "Physics nobel prize"],
             [4, "Sociology", "Sociology nobel prize"],
             ]
         )
-    handler.table_add_records(table_nobel, records)
+    handler.table_add_records(tablename="nobelprizes", records=records)
     print(f"creating multiple records")
-    print_records(table_nobel.records)
+    print_records(handler.table_read_records(tablename="nobelprizes"))
 
     # adding multiple records in one go
     records = handler.records_create(
-        table_papers,
-            [
+        tablename="papers",
+        recordsvalues=[
             [1, "Palestine", "Extrapolation on the palestinian cause"],
             [2, "Wealth", "On the Wealth of Nations"],
             [3, "Time", "A brief history of time"],
             [4, "Fear", "Controlling your fear"],
             ]
         )
-    handler.table_add_records(table_papers, records)
+    handler.table_add_records(tablename="papers", records=records)
     print(f"creating multiple records")
-    print_records(table_papers.records)
+    print_records(handler.table_read_records(tablename="papers"))
 
     # adding a crossref table
     crossref_table = handler.crossref_create(
@@ -501,15 +523,15 @@ if __name__ == "__main__":
     print(f"Database contains {handler.database.tables}")
 
     # get crossreferences
-    crossreferences = handler.crossref_get_all("scientists")
-    print(f"Table {table_scientists.name} has links to:")
+    crossreferences = handler.crossref_get_all(tablename="scientists")
+    print(f"Table scientists has links to:")
     for crossreference in crossreferences:
         print(f"- {crossreference.name}")
 
     # adding a crossref
     handler.crossref_add_record(
-        table1=table_scientists,
-        table2=table_nobel,
+        tablename1="scientists",
+        tablename2="nobelprizes",
         where1=[
             ["name", ["Hawking"]]
             ],
@@ -520,8 +542,8 @@ if __name__ == "__main__":
 
     # adding a crossref
     handler.crossref_add_record(
-        table1=table_scientists,
-        table2=table_nobel,
+        tablename1="scientists",
+        tablename2="nobelprizes",
         where1=[
             ["name", ["Einstein"]]
             ],
@@ -533,8 +555,8 @@ if __name__ == "__main__":
 
     # adding crossreff based upon primary keys
     handler.crossref_add_record(
-        table1=table_scientists,
-        table2=table_papers,
+        tablename1="scientists",
+        tablename2="papers",
         where1=[1],
         where2=[3, 4],
         description="Hawking wrote papers about time and fear of time"
@@ -562,8 +584,8 @@ if __name__ == "__main__":
 
     # gathering all records of all tables
     recordset = []
-    for key in handler.database.tables:
-        records = handler.table_read_records(handler.database.tables[key])
+    for tablename in handler.database.tables:
+        records = handler.table_read_records(tablename)
         recordset += [records]
     
     # printing all the records of all tables
